@@ -732,3 +732,8 @@ class Storage(base.Storage):
             if isinstance(objects, (tuple, list))
             else objects.key
         )
+
+    def statObject(self, key, bucketOrBucketName=None):
+        bucket = self.getBucket(bucketOrBucketName)
+        stat = bucket.head_object(key)
+        return base.ObjectStat(stat, 'oss')
