@@ -6,15 +6,13 @@ import os
 import sys
 import traceback
 import uuid
-import gc
 import logging
 import gevent
 import gevent.pywsgi
 import gevent.util
+import copy
 import socketio
 from engineio import static_files
-from gevent._greenlet_primitives import get_reachable_greenlets
-
 from suanpan import asyncio, error, g, runtime, utils
 from suanpan.arguments import Bool, BoolOrInt, Float, Int, String
 from suanpan.components import Arguments
@@ -30,6 +28,7 @@ from suanpan.stream.objects import Context
 from suanpan.stream.sio import StreamServer, WebSocketHandler
 from suanpan.utils import functional, json, port
 from suanpan.path import rootDir
+from suanpan.objects import Context
 
 
 class Stream(
@@ -52,6 +51,7 @@ class Stream(
         String("stream-sio-static-url-prefix", default="/"),
         String("stream-app-event-hub", default=f"app-event-hub-{g.appId}"),
         Int("stream-app-event-hub-max-length", default=1000),
+        Bool("stream-enable-ergo", default=False),
     ]
     STREAM_PARAMS_FILE = "params.json"
 
@@ -68,10 +68,11 @@ class Stream(
         self.sioLoop = StreamSIOLoop(self)
         self.httpLoop = StreamHTTPLoop(self)
         self.pubsubLoop = StreamPubsubLoop(self)
+        self.ergoLoop = StreamErgoLoop(self)
 
     @property
     def loops(self):
-        return [self.callLoop, self.triggerLoop, self.sioLoop, self.httpLoop, self.pubsubLoop]
+        return [self.callLoop, self.triggerLoop, self.sioLoop, self.httpLoop, self.pubsubLoop, self.ergoLoop]
 
     def beforeInit(self):
         logger.logDebugInfo()
@@ -671,3 +672,60 @@ class StreamPubsubLoop(StreamBaseLoop):
             maxlen=self.stream.options["stream-app-event-hub-max-length"],
             trimImmediately=self.stream.options["stream-send-queue-trim-immediately"],
         )
+
+
+class StreamErgoLoop(StreamMQBaseLoop):
+    DEFAULT_HANDLER_KEY = "call"
+    DEFAULT_HANDLER_CLASS = handlers.CallHandler
+
+    def __init__(self, stream):
+        super().__init__(stream)
+
+    @property
+    def ready(self):
+        try:
+            from . import ergo
+        except ImportError:
+            logger.warning("failed to import the ergo.so")
+            return False
+        return self.stream.options["stream-enable-ergo"] or g.serviceErgoEnable
+
+    def close(self):
+        from . import ergo
+        logger.debug("Stream Ergo Loop Closing...")
+        ergo.close()
+        logger.debug("Stream Ergo Loop Closed")
+
+    def getHandler(self):
+        return self.stream.callLoop.getHandler()
+
+    def _ergo_send(self, result, message_id, output):
+        from . import ergo
+        logger.debug(utils.shorten(f"Send to ergo {output}: {result}", self.DEFAULT_LOGGER_MAX_LENGTH))
+        if isinstance(result, str):
+            result = result.encode('utf8')
+        ergo.send(result, message_id, output)
+
+    def ergo_send(self, results, message=None, args=None, **_):
+        if not self.ready:
+            raise Exception('ergo is not enabled')
+
+        message = message or self.generateMessage()
+        if not args:
+            args = copy.deepcopy(self.getHandler().getArguments(include="outputs"))
+        for arg in args:
+            arg.use_argo = True
+        outputs = self.getHandler().save(results, args=args, message=message)
+        for key, value in outputs.items():
+            self._ergo_send(value, message["id"], key)
+
+    @runtime.globalrun
+    def start(self):
+        from . import ergo
+
+        ergo.init()
+        logger.debug("Stream Ergo Loop Starting...")
+        while True:
+            msg, msg_id, in_port = ergo.recv()
+            message = {'id': msg_id, in_port: msg}
+            self.handle(message)
